from datetime import datetime, timezone
from typing import Optional, List

from pydantic import BaseModel, field_validator


class Competition(BaseModel):
    """Represents the data about competition which provide the judobase api"""

    id_competition: str
    date_from: str
    date_to: str
    name: str
    has_results: int
    city: str
    street: str
    street_no: str
    comp_year: int
    prime_event: bool
    continent_short: str
    has_logo: bool
    competition_code: Optional[str]
    updated_at_ts: datetime
    updated_at: datetime
    timezone: Optional[str]
    id_live_theme: int
    code_live_theme: str
    country_short: str
    country: str
    id_country: int
    is_teams: int
    status: Optional[str]
    external_id: Optional[str]
    id_draw_type: int
    ages: List[str]
    rank_name: Optional[str]

    @field_validator('updated_at', mode='after')
    @classmethod
    def parse_updated_at(cls, value):
        return value.replace(tzinfo=timezone.utc)

    @field_validator('date_from', mode='after')
    @classmethod
    def parse_date_from(cls, value):
        return datetime.strptime(value, '%Y/%m/%d')

    @field_validator('date_to', mode='after')
    @classmethod
    def parse_date_to(cls, value):
        if isinstance(value, str):
            return datetime.strptime(value, '%Y/%m/%d')


class Contest(BaseModel):
    """Represents the data about contest which provide the judobase api"""

    # general contest data
    id_competition: str
    id_fight: str
    id_person_blue: str
    id_person_white: str
    id_winner: Optional[str]
    is_finished: bool
    round: int
    duration: Optional[str]
    gs: bool
    bye: str
    fight_duration: Optional[str]
    weight: Optional[str]
    id_weight: Optional[str]
    type: int
    round_code: Optional[str]
    round_name: str
    mat: int
    date_start_ts: datetime
    updated_at: datetime
    first_hajime_at_ts: datetime

    # white person details
    ippon_w: Optional[int]
    waza_w: Optional[int]
    yuko_w: Optional[int]
    penalty_w: Optional[int]
    hsk_w: Optional[int]
    person_white: str
    id_ijf_white: str
    family_name_white: str
    given_name_white: str
    timestamp_version_white: str
    country_white: Optional[str]
    country_short_white: Optional[str]
    id_country_white: Optional[str]
    picture_folder_1: Optional[str]
    picture_filename_1: Optional[str]
    personal_picture_white: Optional[str]

    # blue person details
    ippon_b: Optional[int]
    waza_b: Optional[int]
    yuko_b: Optional[int]
    penalty_b: Optional[int]
    hsk_b: Optional[int]
    person_blue: str
    id_ijf_blue: str
    family_name_blue: str
    given_name_blue: str
    timestamp_version_blue: str
    country_blue: Optional[str]
    country_short_blue: Optional[str]
    id_country_blue: Optional[str]
    picture_folder_2: Optional[str]
    picture_filename_2: Optional[str]
    personal_picture_blue: Optional[str]

    # competitions details
    competition_name: str
    external_id: str
    city: str
    age: Optional[str]
    rank_name: Optional[str]
    competition_date: str
    date_raw: str
    comp_year: str

    # other details
    tagged: int
    kodokan_tagged: int
    published: str
    sc_countdown_offset: int
    fight_no: int
    contest_code_long: str
    media: Optional[str]
    id_competition_teams: Optional[str]
    id_fight_team: Optional[str]


    @field_validator('updated_at', mode='after')
    @classmethod
    def parse_updated_at(cls, value):
        return value.replace(tzinfo=timezone.utc)

    @field_validator('date_start_ts', mode='after')
    @classmethod
    def parse_date_start_ts(cls, value):
        return value.replace(tzinfo=timezone.utc)

    @field_validator('first_hajime_at_ts', mode='after')
    @classmethod
    def parse_first_hajime_at_ts(cls, value):
        return value.replace(tzinfo=timezone.utc)


class Competitor(BaseModel):
    """Represents the data about competitor which provide the judobase api"""

    family_name: str
    middle_name: Optional[str]
    given_name: str
    family_name_local: str
    middle_name_local: Optional[str]
    given_name_local: str
    short_name: Optional[str]
    gender: str
    folder: str
    picture_filename: str
    ftechique: Optional[str]
    side: str
    coach: str
    best_result: str
    height: str
    birth_date: datetime
    country: str
    id_country: str
    country_short: str
    file_flag: Optional[str]
    club: Optional[str]
    belt: Optional[str]
    youtube_links: Optional[str]
    status: Optional[str]
    archived: Optional[str]
    categories: List[str]
    dob_year: Optional[str]
    age: Optional[str]
    death_age: Optional[str]
    personal_picture: str



import asyncio
from datetime import datetime

from .base import _Base
from .schemas import Competition, Contest


class JudoBase(_Base):

    async def get_all_comps(self) -> list[Competition]:
        """Returns data for all competitions."""

        return await self._get_competition_list()

    async def get_all_contests(self) -> list[Contest]:
        """Returns data for all contests using concurrent calls to _find_contests."""

        comps = await self.get_all_comps()
        tasks = [self._find_contests(c.id_competition) for c in comps]

        results = await asyncio.gather(*tasks)

        contests = [contest for sublist in results for contest in sublist]
        return contests

    async def get_comps_in_range(
        self,
        start_date: datetime,
        end_date: datetime
    ) -> list[Competition]:
        """Returns data for competitions in specified date range."""

        all_comps = await self.get_all_comps()
        return [c for c in all_comps if start_date <= c.date_from <= end_date]



import pytest
from aioresponses import aioresponses
from aiohttp import ClientSession

from judobase.base import _Base
from judobase.schemas import Competition, Competitor, Contest


@pytest.mark.asyncio
async def test_aenter_aexit():
    """Test that the context manager properly creates and closes the session."""

    async with _Base() as client:
        assert isinstance(client._session, ClientSession)
        assert not client._session.closed
    assert client._session.closed


@pytest.mark.asyncio
async def test_get_json_success():
    """Test _get_json with a successful API response."""

    async with _Base() as client:
        params = {"key": "value"}
        mock_response = {"data": "test"}

        with aioresponses() as mock:
            mock.get(client.base_url + "get_json", payload=mock_response, status=200)

            result = await client._get_json(params)
            assert result == mock_response


@pytest.mark.asyncio
async def test_get_json_failure():
    """Test _get_json with a failed API response."""

    async with _Base() as client:
        params = {"key": "value"}

        with aioresponses() as mock:
            mock.get(client.base_url + "get_json", status=500)

            with pytest.raises(ConnectionError) as exc_info:
                await client._get_json(params)

            assert "500" in str(exc_info.value)


@pytest.mark.asyncio
async def test_get_competition_list():
    """Test _get_competition_list method."""

    async with _Base() as client:
        params = {
            "params[action]": "competition.get_list",
            "params[year]": "2023",
            "params[month]": "01",
            "params[sort]": -1,
            "params[limit]": 5000,
        }

        mock_response = [{"id": 1, "name": "Competition 1"}]
        expected_result = [Competition(id=1, name="Competition 1")]

        with aioresponses() as mock:
            mock.get(client.base_url + "get_json", payload=mock_response, status=200)

            result = await client._get_competition_list(years="2023", months="01")
            assert result == expected_result


@pytest.mark.asyncio
async def test_find_contests():
    """Test _find_contests method."""

    async with _Base() as client:
        params = {
            "params[action]": "contest.find",
            "params[id_competition]": "123",
            "params[id_weight]": "",
            "params[id_person]": "",
            "params[order_by]": "cnum",
            "params[limit]": 5000,
        }

        mock_response = {"contests": [{"id": 1, "name": "Contest 1"}]}
        expected_result = [Contest(id=1, name="Contest 1")]

        with aioresponses() as mock:
            mock.get(client.base_url + "get_json", payload=mock_response, status=200)

            result = await client._find_contests(id_competition="123")
            assert result == expected_result


@pytest.mark.asyncio
async def test_competitor_info():
    """Test competitor_info method."""

    async with _Base() as client:
        params = {
            "params[action]": "competitor.info",
            "params[id_person]": "456",
        }

        mock_response = [{"id": 456, "name": "Competitor 1"}]
        expected_result = [Competitor(id=456, name="Competitor 1")]

        with aioresponses() as mock:
            mock.get(client.base_url + "get_json", payload=mock_response, status=200)

            result = await client.competitor_info(id_competitor="456")
            assert result == expected_result



from aiohttp import ClientSession

from judobase.schemas import Competition, Contest, Competitor


class _Base:
    """Represents basic judobase API functionality."""

    def __init__(self):
        self.base_url = "https://data.ijf.org/api/"
        self._session = None

    async def __aenter__(self) -> "_Base":
        """Enter the async context and create a session."""

        self._session = ClientSession()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        """Exit the async context and close the session."""

        if self._session and not self._session.closed:
            await self._session.close()

    async def _get_json(self, params) -> dict:
        """Makes get request to specified API and returns JSON."""

        response = await self._session.get(
            self.base_url + "get_json",
            timeout=10,
            params=params,
        )

        if response.status != 200:
            raise ConnectionError(f"{response.status}")

        return await response.json()

    async def _get_competition_list(self, years: str = "", months: str = "") -> list[Competition]:
        """Returns competition list by specified filters."""

        params = {
            "params[action]": "competition.get_list",
            "params[year]": years,
            "params[month]": months,
            "params[sort]": -1,
            "params[limit]": 5000,
        }
        return [Competition(**item) for item in await self._get_json(params)]

    async def _find_contests(
        self,
        id_competition: str = "",
        id_weight: str = "",
        id_person: str = ""
    ) -> list[Contest]:
        """Returns contest list by specified filters."""

        params = {
            "params[action]": "contest.find",
            "params[id_competition]": id_competition,
            "params[id_weight]": id_weight,
            "params[id_person]": id_person,
            "params[order_by]": "cnum",
            "params[limit]": 5000,
        }
        result = await self._get_json(params)
        return [Contest(**item) for item in result["contests"]]

    async def competitor_info(self, id_competitor: str = ""):
        """Returns info about competitor by specified filters."""

        params = {
            "params[action]": "competitor.info",
            "params[id_person]": id_competitor,
        }
        return [Competitor(**item) for item in await self._get_json(params)]



from setuptools import setup, find_packages


package = "judobase"
requirements = [
    "aioresponses>=0.7.7",
    "aiohttp>=3.11.11",
    "pydantic>=2.10.5",
    "setuptools>=75.6.0"
]
test_requirements = [
    "pytest>=8.3.4",
]


setup(
    name=package,
    version="0.1.0",
    author="ddzgoev",
    author_email="ddzgoev@gmail.com",
    description="Python Judobase API async client",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/DavidDzgoev/judobase",
    packages=[
        'judobase',
    ],
    package_dir={'judobase': 'judobase'},
    license="MIT",
    zip_safe=False,
    keywords='judobase,wrapper,client,async,api',
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
    install_requires=requirements,
    tests_require=test_requirements,
)
